/*
 * Copyright 2010 Aleksey Shipilev
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.free.dedup;

import java.io.File;
import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.BlockingQueue;

public class EnumeratorWorker implements Runnable {
    private final File root;
    private final BlockingQueue<File> processQueue;

    public EnumeratorWorker(File root, BlockingQueue<File> resultQueue) {
        this.root = root;
        this.processQueue = resultQueue;
    }

    @Override
    public void run() {

        Queue<File> files = new LinkedList<File>();
        files.add(root);

        while (files.size() > 0) {
            File file = files.poll();
            File[] children = file.listFiles();
            if (children != null) {
                for (File child : children) {
                    files.add(child);
                }
            }
            if (file.isFile()) {
                try {
                    processQueue.put(file);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    return;
                }
            }
        }
    }
}
